from time import time
from .all_tests import test_list

class tests:
    def __init__(self, func_to_test):
        test_list.append(self)
        self.func = func_to_test        
        self.result = None
        self.time_taken = None
        self.error = None
    
    def __call__(self, func_that_tests):
        self.test = func_that_tests
        return self.do_test

    def do_test(self):
        t_start = time()
        try:
            self.test(self.func)
        except Exception as e:
            self.result = False
            self.time_taken = None
            self.error = e
        else:
            self.result = True
            self.time_taken = time() - t_start
            self.error = None
        return self

    def __repr__(self):
        lines = []
        lines.append("Test for '{}':".format(self.func.__name__))
        if self.test.__doc__ is not None:
            lines.append("\t{0}".format(self.test.__doc__))
        lines.append("\tResult: {}".format(self.result))
        if self.result is True:
            lines.append("\tTook {} seconds".format(self.time_taken))
            lines.append("\tTest Passed.")
        else:
            lines.append("\tWith error:")
            lines.append("\t\t{}".format(self.error.__class__.__name__))
            lines.append("\t\t{}".format(self.error.__str__()))
            lines.append("\tTest Failed.")
        return "\n".join(lines)        


test_list = []

def test_all(quiet_mode = False):
    lines = []
    count = len(test_list)
    lines.append("Test count: {}".format(count))
    failed = 0
    for test in test_list:
        test.do_test()
        if test.result:
            if quiet_mode:
                continue
        else:
            failed += 1
            lines.append("################IMPORTANT################")
        lines.append(str(test))
    lines.append("Passed: {0}/{1}, Failed {2}/{1}".format(count - failed, count, failed))
    if failed > 0:
        lines.append("Overall test failed.")
    else:
        lines.append("Overall test passed.")
    return "\n".join(lines)
        
        

        